﻿; GetWindowHandle.ahk[UTF-8] (2025/01/25 22:00)
; for AutoHotKey v2.0.19 or later
;
; 指定のウィンドウが一意に絞り込めるかどうか(他のツール類を使う前にテストするなど)確認します
;    GetWindowHandle.exe Mode Target (ArrowStyle) (DenyStyle)
;       Mode
;          1/exit/hwnd : 結果を復帰コードで返します
;          2/disp/dialog : ダイアログボックスで結果を表示し復帰コードも返します
;       Target
;          ウィンドウ文字列/ahk_exe フルパス/ahk_class クラスID など両端""でAutoHotKey
;          独自のウィンドウ指定記述、現在アクティブなウィンドウを指定する場合は"A"
;          https://ahkscript.github.io/ja/docs/v2/misc/WinTitle.htm
;          対象ウィンドウのウィンドウスタイルは WS_VISIBLE ビットが有効で WS_DISABLED ビットが無効の
;          ものとしています。複数ウィンドウに該当する場合はエラー表示し処理しません。
;          マッチを正規表現で記述したい時は先頭に"REGEX:"を書きます
;          AutoHotKeyV2の正規表現文法は下記参照
;          https://ahkscript.github.io/ja/docs/v2/misc/RegEx-QuickRef.htm
;          ex) "ahk_exe C:\Program Files\Mozilla Firefox\firefox.exe"
;          ex) "ahk_exe C:\Program Files\Vivaldi\Application\vivaldi.exe ahk_class Chrome_WidgetWin_1"
;       ArrowStyle
;          処理対象であるウィンドウスタイルのビット和(andをとって0以外になれば対象)の16進表記
;          省略された場合やdef/defaultと書かれた時は 0x10000000(WS_VISIBLE) が指定されたものとします
;          全てを対象外にしてしまう 0x00000000 は指定できません
;       DenyStyle
;          処理対象から除外するウィンドウスタイルのビット和(andをとって0になれば対象)の16進表記
;          省略された場合やdef/defaultと書かれた時は 0x08000000(WS_DISABLED) が指定されたものとします
;          全てを対象外にしてしまう 0xFFFFFFFF は指定できません
;
;       戻り値(exit-code)　※バッチファイルで本コマンド実行直後に %errorlevel% で参照出来ます
;          一意のウィンドウが該当した場合、見つかったウィンドウの hWnd
;          条件に合致するウィンドウが複数見つかった場合は見つかった数にマイナスを掛けた数値
;          見つからない場合は 0


WS_VISIBLE      := 0x10000000
WS_DISABLED     := 0x08000000

/* 同時に複数実行を許可する */
#SingleInstance Off

#WinActivateForce
/* 非表示のウィンドウも捜索対象としておかないと仮想デスクトップで動作しない */
DetectHiddenWindows True


/* コマンドオプション数チェック */
if( A_Args.Length < 2 ){
	DispUsage()
}

/* モードとウィンドウ指定 */
iMode := A_Args[1]
strTarget := A_Args[2]
arg_regx := 2
if( StrUpper(SubStr(strTarget,1,6)) = "REGEX:" ){
	arg_regx := "RegEx"
	strTarget := SubStr(strTarget,7)
}
UniqueID := 0
iArrowStyle := 0
iDenyStyle := 0xFFFFFFFF

/* モードの扱い */
if( iMode = 1 or StrUpper(iMode) = "EXIT" or StrUpper(iMode) = "HWND" ){
	iMode := 1
}else if( iMode = 0 or iMode = 2 or StrUpper(iMode) = "DISP" or StrUpper(iMode) = "DIALOG" ){
	iMode := 0
}else{
	DispUsage()
}

/* 必須ウィンドウスタイルの扱い */
if( A_Args.Length >= 3 ){
	iArrowStyle := A_Args[3]
}
if( iArrowStyle = 0 or StrUpper(iArrowStyle) = "DEF" or StrUpper(iArrowStyle) = "DEFAULT" ){
	iArrowStyle := WS_VISIBLE
}else{
	if( IsNumber(iArrowStyle) ){
		iArrowStyle := Number(iArrowStyle)
	}
}

/* 除外ウィンドウスタイルの扱い */
if( A_Args.Length >= 4 ){
	iDenyStyle := A_Args[4]
}
if( iDenyStyle = 0xFFFFFFFF or StrUpper(iDenyStyle) = "DEF" or StrUpper(iDenyStyle) = "DEFAULT" ){
	iDenyStyle := WS_DISABLED
}else{
	if( IsNumber(iDenyStyle) ){
		iDenyStyle := Number(iDenyStyle)
	}
}


/* タイトルマッチモードの設定(2=部分一致, "RegEx"=正規表現) */
SetTitleMatchMode(arg_regx)

/* フルパスに合致するウィンドウを列挙(非表示ウィンドウも対象としているため沢山ヒットする) */
listWin := WinGetList(strTarget)
idxWin := 0
findCount := 0

/* ウィンドウスタイルでマスク */
for hWin in listWin{
	idxWin := idxWin + 1
	try{
		style := WinGetStyle("ahk_id " hWin)
		if( style & iArrowStyle and not(style & iDenyStyle) ){
			findCount := findCount + 1
			UniqueID := hWin
		}
	}
}

if( UniqueID = 0 ){
	if( iMode = 0 ){
		MsgBox(strTarget " not found.")
	}
	ExitApp(0)
}

if( findCount != 1 ){
	if( iMode = 0 ){
		MsgBox(strTarget " has " findCount " windows.")
	}
	ExitApp( Number(-1 * findCount) )
}

if( iMode = 0 ){
	MsgBox(strTarget " found hWnd[" UniqueID "].")
}

ExitApp( UniqueID )

DispUsage(){
	msg := 'Usage:`n'
	msg := msg 'GetWindowHandle.exe Mode Target (...options)`n`n'
	msg := msg '[Mode]`n'
	msg := msg '    1/exit/hwnd = (exit-code)`n'
	msg := msg '      Unique-window -> hwnd`n'
	msg := msg '      Not found -> 0`n'
	msg := msg '      Multi-windows -> count * (-1)`n'
	msg := msg '    2/disp/dialog = Display DialogBox`n'
	msg := msg '[Target]`n'
	msg := msg '   "window-title / REGEX:match-title-str`n'
	msg := msg '    ahk_exe full-path`n'
	msg := msg '    ahk_class class-name..."`n'
	msg := msg '[ArrowStyle]`n'
	msg := msg '    enabled WindowStyle Bits`n'
	msg := msg '    def/default = 0x10000000(WS_VISIBLE)`n'
	msg := msg '    ex) 0x16C70000`n'
	msg := msg '[DenyStyle]`n'
	msg := msg '    disabled WindowStyleBits`n'
	msg := msg '    def/default = 0x08000000(WS_DISABLED)`n'
	msg := msg '    ex) 0xE8000000`n'
	msg := msg '[DispError]`n'
	msg := msg '    1/on/disp = display ErrorDialog`n'
	msg := msg '[DebugMode]`n'
	msg := msg '    1/on/debug = Debug mode`n'
	MsgBox(msg)
	ExitApp(1)
}


